"use strict";

var pupilmax = 9.0;

function set_pupilsize(gl, luminance)
{
  var p = pupilmax - 2.5*(1 + Math.tanh(0.4*(Math.log(luminance*Math.PI*1e-1) + 0.5)));
  gl.uniform1f(gl.getUniformLocation(gl.program, "pupilsize"), p);
}

window.onload = function init()
{
  var canvas = document.getElementById("webgl");

  var gl = WebGLUtils.setupWebGL(canvas);
  if (!gl) {
    console.log('Failed to get the rendering context for WebGL');
    return;
  }
  
  // Load shaders and initialize uniforms
  gl.program = initShaders(gl, "vertex-shader", "fragment-shader");
  gl.useProgram(gl.program);
  var P = ortho(-1, 1, -2.15, 9.65, -1, 1);
  gl.uniformMatrix4fv(gl.getUniformLocation(gl.program, "projectionMatrix"), false, flatten(P));
  gl.uniform1f(gl.getUniformLocation(gl.program, "pupilmax"), pupilmax);
  var luminance = parseFloat(document.getElementById("luminance").value);
  set_pupilsize(gl, luminance);
  gl.clearColor(1.0, 1.0, 1.0, 1.0);

  // Initialize scene geometry
  var n = initVertexBuffers(gl, canvas.width);

  // Register the event handler
  document.getElementById("configure").onclick = function ()
  {
    luminance = parseFloat(document.getElementById("luminance").value);
    set_pupilsize(gl, luminance);
  }
  document.onkeypress = function (ev)
  { // Key is pressed
    switch(ev.keyCode) {
      case 13: // enter
        luminance = parseFloat(document.getElementById("luminance").value);
        set_pupilsize(gl, luminance);
        document.body.focus();
        break;
    }
  };
  var animate = true;
  document.getElementById("toggle").onclick = function () { animate = !animate; if(animate) requestAnimFrame(tick); }


  function tick(timestamp)
  {
    gl.uniform1f(gl.getUniformLocation(gl.program, "time"), timestamp/2000);
    render(gl, n);
    if(animate)
      requestAnimFrame(tick);
  }
  tick(0);
}

function render(gl, no_of_verts)
{
  gl.clear(gl.COLOR_BUFFER_BIT);
  gl.drawArrays(gl.LINE_STRIP, 0, no_of_verts);
}

function initVertexBuffers(gl, width)
{
  // Create a curve of line segments
  var vertices = new Float32Array(width*2);

  for(var i = 0; i < width; ++i)
  {
    vertices[2*i] = 2*i/width - 1;
    vertices[2*i + 1] = 0;
  }
  var vBuffer = gl.createBuffer();
  gl.bindBuffer(gl.ARRAY_BUFFER, vBuffer);
  gl.bufferData(gl.ARRAY_BUFFER, vertices, gl.STATIC_DRAW);

  var vPosition = gl.getAttribLocation(gl.program, "vPosition");
  gl.vertexAttribPointer(vPosition, 2, gl.FLOAT, false, 0, 0);
  gl.enableVertexAttribArray(vPosition);

  return width;
}
